# Copyright (c) 2016 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
import copy
import six
from six.moves.urllib import parse


def serialize(obj):
    """Serialize the given object
    :param obj: string representation of the object
    :return: encoded object (its type is unicode string)
    """
    result = base64.urlsafe_b64encode(obj)
    # this workaround is needed because in case of python 3 the
    # urlsafe_b64encode method returns string of 'bytes' class.
    if six.PY3:
        result = result.decode()
    return result


def deserialize(obj):
    """Deserialize the given object
    :param obj: string representation of the encoded object
    :return: decoded object (its type is unicode string)
    """
    result = base64.urlsafe_b64decode(obj)
    # this workaround is needed because in case of python 3 the
    # urlsafe_b64decode method returns string of 'bytes' class
    if six.PY3:
        result = result.decode()
    return result


def delete_pagination_params_from_request(request, save_limit=None):
    """Delete marker and limit parameters from GET requests
    :param request: instance of GET request
    :param save_limit: if True, 'limit' will not be deleted
    :return: instance of GET request without marker or limit
    """
    request = copy.copy(request)
    request.GET = request.GET.copy()

    params = ['marker']
    if not save_limit:
        params.append('limit')

    for param in ['marker', 'limit']:
        if param in request.GET:
            del(request.GET[param])
            query_string = request.META.get('QUERY_STRING', '')
            query_dict = parse.parse_qs(query_string)
            if param in query_dict:
                del(query_dict[param])

            query_string = parse.urlencode(query_dict, doseq=True)
            request.META['QUERY_STRING'] = query_string
    return request


def smart_sort_helper(version):
    """Allows intelligent sorting of plugin versions, for example when
    minor version of a plugin is 11, sort numerically so that 11 > 10,
    instead of alphabetically so that 2 > 11
    """
    def _safe_cast_to_int(obj):
        try:
            return int(obj)
        except ValueError:
            return obj
    return [_safe_cast_to_int(part) for part in version.split('.')]



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions

from sahara_dashboard.api import sahara as saharaclient
import sahara_dashboard.content.data_processing.clusters. \
    nodegroup_templates.workflows.create as create_flow


class CopyNodegroupTemplate(create_flow.ConfigureNodegroupTemplate):
    success_message = _("Node Group Template copy %s created")

    def __init__(self, request, context_seed, entry_point, *args, **kwargs):
        self.template_id = context_seed["template_id"]
        self.template = saharaclient.nodegroup_template_get(request,
                                                            self.template_id)
        self._set_configs_to_copy(self.template.node_configs)

        plugin = self.template.plugin_name
        hadoop_version = self.template.hadoop_version

        request.GET = request.GET.copy()
        request.GET.update(
            {"plugin_name": plugin, "hadoop_version": hadoop_version})

        super(CopyNodegroupTemplate, self).__init__(request, context_seed,
                                                    entry_point, *args,
                                                    **kwargs)

        g_fields = None
        snp_fields = None
        s_fields = None
        share_fields = None
        for step in self.steps:
            if isinstance(step, create_flow.GeneralConfig):
                g_fields = step.action.fields
            if isinstance(step, create_flow.SecurityConfig):
                s_fields = step.action.fields
            if isinstance(step, create_flow.SelectNodeProcesses):
                snp_fields = step.action.fields
            if isinstance(step, create_flow.SelectNodeGroupShares):
                share_fields = step.action.fields

        g_fields["nodegroup_name"].initial = self.template.name + "-copy"
        g_fields["description"].initial = self.template.description
        g_fields["flavor"].initial = self.template.flavor_id

        if hasattr(self.template, "availability_zone"):
            g_fields["availability_zone"].initial = (
                self.template.availability_zone)

        if hasattr(self.template, "volumes_availability_zone"):
            g_fields["volumes_availability_zone"].initial = \
                self.template.volumes_availability_zone

        storage = "cinder_volume" if self.template.volumes_per_node > 0 \
            else "ephemeral_drive"
        volumes_per_node = self.template.volumes_per_node
        volumes_size = self.template.volumes_size
        volume_type = self.template.volume_type
        volume_local_to_instance = self.template.volume_local_to_instance
        g_fields["storage"].initial = storage
        g_fields["volumes_per_node"].initial = volumes_per_node
        g_fields["volumes_size"].initial = volumes_size
        g_fields["volumes_availability_zone"].initial = \
            self.template.volumes_availability_zone
        g_fields['volume_type'].initial = volume_type
        g_fields['volume_local_to_instance'].initial = volume_local_to_instance
        g_fields["proxygateway"].initial = self.template.is_proxy_gateway
        g_fields["use_autoconfig"].initial = self.template.use_autoconfig
        g_fields["is_public"].initial = self.template.is_public
        g_fields['is_protected'].initial = self.template.is_protected
        g_fields["image"].initial = self.template.image_id

        if self.template.floating_ip_pool:
            g_fields['floating_ip_pool'].initial = (
                self.template.floating_ip_pool)

        s_fields["security_autogroup"].initial = (
            self.template.auto_security_group)

        if self.template.security_groups:
            s_fields["security_groups"].initial = dict(
                [(sg, sg) for sg in self.template.security_groups])

        processes_dict = dict()
        try:
            plugin_details = saharaclient.plugin_get_version_details(
                request,
                plugin,
                hadoop_version)
            plugin_node_processes = plugin_details.node_processes
        except Exception:
            plugin_node_processes = dict()
            exceptions.handle(request,
                              _("Unable to fetch plugin details."))
        for process in self.template.node_processes:
            # need to know the service
            _service = None
            for service, processes in plugin_node_processes.items():
                if process in processes:
                    _service = service
                    break
            processes_dict["%s:%s" % (_service, process)] = process
        snp_fields["processes"].initial = processes_dict

        if share_fields:
            share_fields["shares"].initial = (
                self._get_share_defaults(share_fields))

    def _get_share_defaults(self, share_fields):
        values = dict()
        choices = share_fields['shares'].choices
        for i, choice in enumerate(choices):
            share_id = choice[0]
            s = filter(lambda s: s['id'] == share_id, self.template.shares)
            if len(s) > 0:
                path = s[0].get('path', '')
                values["share_id_{0}".format(i)] = {
                    "id": s[0]["id"],
                    "path": path,
                    "access_level": s[0]["access_level"]
                }
            else:
                values["share_id_{0}".format(i)] = {
                    "id": None,
                    "path": None,
                    "access_level": None
                }
        return values



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests import helpers
from openstack_dashboard.test.integration_tests.regions import messages

# Size of binary name is limited to 50 characters
BINARY_NAME = helpers.gen_random_resource_name(resource='jobbinary',
                                               timestamp=False)[0:50]
SCRIPT_NAME = helpers.gen_random_resource_name(resource='scriptname',
                                               timestamp=False)


class TestSaharaJobBinary(helpers.TestCase):

    def _sahara_create_delete_job_binary(self, job_name):
        # create job binary
        job_binary_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobbinariespage())
        self.assertFalse(job_binary_pg.is_job_binary_present(job_name),
                         "Job binary was present in the binaries table"
                         " before its creation.")
        job_binary_pg.create_job_binary(job_name, SCRIPT_NAME)

        # verify that job is created without problems
        self.assertTrue(
            job_binary_pg.find_message_and_dismiss(messages.SUCCESS))
        self.assertFalse(
            job_binary_pg.find_message_and_dismiss(messages.ERROR),
            "Error message occurred during binary job creation.")
        self.assertTrue(job_binary_pg.is_job_binary_present(job_name),
                        "Job binary is not in the binaries job table after"
                        " its creation.")

        # delete binary job
        job_binary_pg.delete_job_binary(job_name)

        # verify that job was successfully deleted
        self.assertTrue(
            job_binary_pg.find_message_and_dismiss(messages.SUCCESS))
        self.assertFalse(
            job_binary_pg.find_message_and_dismiss(messages.ERROR),
            "Error message occurred during binary job deletion.")
        self.assertFalse(job_binary_pg.is_job_binary_present(job_name),
                         "Job binary was not removed from binaries job table.")

    def test_sahara_create_delete_job_binary_internaldb(self):
        """Test the creation of a Job Binary in the Internal DB."""
        self._sahara_create_delete_job_binary(BINARY_NAME)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from datetime import datetime

from oslo_config import cfg

from openstack_dashboard.test.integration_tests import helpers


class SaharaTestCase(helpers.AdminTestCase):

    @classmethod
    def setUpClass(cls):
        sahara_group = cfg.OptGroup(
            'sahara', help='Sahara specific tests config group')
        cls.CONFIG.register_group(sahara_group)
        fake_image_location = cfg.StrOpt('fake_image_location')
        cls.CONFIG.register_opt(fake_image_location, group=sahara_group)
        ssh_user = cfg.StrOpt('fake_image_ssh_user')
        cls.CONFIG.register_opt(ssh_user, group=sahara_group)
        project_name = cfg.StrOpt('project_name', default='demo')
        cls.CONFIG.register_opt(project_name, group=sahara_group)
        ip_pool = cfg.StrOpt('ip_pool', default="public")
        cls.CONFIG.register_opt(ip_pool, group=sahara_group)
        launch_timeout = cfg.IntOpt('launch_timeout', default=1200)
        cls.CONFIG.register_opt(launch_timeout, group=sahara_group)

        flavor_vcpus = cfg.IntOpt('flavor_vcpus', default=1)
        cls.CONFIG.register_opt(flavor_vcpus, group=sahara_group)
        flavor_ram = cfg.IntOpt('flavor_ram', default=256)
        cls.CONFIG.register_opt(flavor_ram, group=sahara_group)
        flavor_root_disk = cfg.IntOpt('flavor_root_disk', default=3)
        cls.CONFIG.register_opt(flavor_root_disk, group=sahara_group)
        flavor_ephemeral_disk = cfg.IntOpt('flavor_ephemeral_disk', default=0)
        cls.CONFIG.register_opt(flavor_ephemeral_disk, group=sahara_group)
        flavor_swap_disk = cfg.IntOpt('flavor_swap_disk', default=0)
        cls.CONFIG.register_opt(flavor_swap_disk, group=sahara_group)

    def setUp(self):
        super(SaharaTestCase, self).setUp()
        self._suffix = datetime.now().strftime('%H-%M-%S-%f')[:12]
        # select project
        driver = self.home_pg.driver
        driver.find_element_by_css_selector('li.dropdown').click()
        driver.find_element_by_link_text(
            self.CONFIG.sahara.project_name).click()

    def gen_name(self, name):
        return '{}-{}'.format(name, self._suffix)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.http import Http404  # noqa
from django.urls import reverse
from django.utils import http
from django.utils.translation import pgettext_lazy
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy
from saharaclient.api import base as api_base

from horizon import messages
from horizon import tables
from horizon.tabs import base as tabs_base

from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing.jobs.job_templates \
    import tables as j_t
from sahara_dashboard.content.data_processing \
    import tables as sahara_table
from sahara_dashboard.content.data_processing.utils \
    import acl as acl_utils
from sahara_dashboard.content.data_processing.utils import helpers


class JobsFilterAction(tables.FilterAction):
    filter_type = "server"
    filter_choices = (('id', _("ID"), True),
                      ('job', _("Job"), True),
                      ('cluster', _("Cluster"), True),
                      ('status', _("Status"), True))


class JobExecutionGuide(tables.LinkAction):
    name = "jobex_guide"
    verbose_name = _("Job Guide")
    url = "horizon:project:data_processing.jobs:jobex_guide"


class DeleteJob(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Job",
            u"Delete Jobs",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Job",
            u"Deleted Jobs",
            count
        )

    def delete(self, request, obj_id):
        saharaclient.job_execution_delete(request, obj_id)


class ReLaunchJobExistingCluster(j_t.ChoosePlugin):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Launch Job",
            u"Launch Jobs",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Launched Job",
            u"Launched Jobs",
            count
        )

    name = "relaunch-job-existing"
    verbose_name = _("Relaunch On Existing Cluster")
    url = "horizon:project:data_processing.jobs:launch-job"
    classes = ('ajax-modal', 'btn-launch')

    def get_link_url(self, datum):
        base_url = reverse(self.url)
        params = http.urlencode({'job_id': datum.job_id,
                                 'job_execution_id': datum.id})
        return "?".join([base_url, params])


class ReLaunchJobNewCluster(ReLaunchJobExistingCluster):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Launch Job",
            u"Launch Jobs",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Launched Job",
            u"Launched Jobs",
            count
        )

    name = "relaunch-job-new"
    verbose_name = _("Relaunch On New Cluster")
    url = "horizon:project:data_processing.jobs:choose-plugin"
    classes = ('ajax-modal', 'btn-launch')


class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, job_execution_id):
        try:
            return saharaclient.job_execution_get(request, job_execution_id)
        except api_base.APIException as e:
            if e.error_code == 404:
                raise Http404
            else:
                messages.error(request, _("Unable to update row"))


class MakePublic(acl_utils.MakePublic):
    def change_rule_method(self, request, datum_id, **update_kwargs):
        saharaclient.job_execution_update(request, datum_id, **update_kwargs)


class MakePrivate(acl_utils.MakePrivate):
    def change_rule_method(self, request, datum_id, **update_kwargs):
        saharaclient.job_execution_update(request, datum_id, **update_kwargs)


class MakeProtected(acl_utils.MakeProtected):
    def change_rule_method(self, request, datum_id, **update_kwargs):
        saharaclient.job_execution_update(request, datum_id, **update_kwargs)


class MakeUnProtected(acl_utils.MakeUnProtected):
    def change_rule_method(self, request, datum_id, **update_kwargs):
        saharaclient.job_execution_update(request, datum_id, **update_kwargs)


class JobsTable(sahara_table.SaharaPaginateTabbedTable):
    tab_name = 'job_tabs%sjobs_tab' % tabs_base.SEPARATOR

    class StatusColumn(tables.Column):
        def get_raw_data(self, datum):
            return datum.info['status']

    class JobNameColumn(tables.Column):
        @staticmethod
        def link(job_execution):
            if job_execution.job_name:
                return reverse("horizon:project:data_processing."
                               "jobs:jt-details",
                               args=(http.urlquote(job_execution.job_id),))
            else:
                # No link should be generated for a deleted Job.
                return None

        def get_data(self, job_execution):
            return job_execution.job_name or _("Not available")

    class ClusterNameColumn(tables.Column):

        @staticmethod
        def link(job_execution):
            if job_execution.cluster_name:
                return reverse(
                    "horizon:project:data_processing.clusters:cluster-details",
                    args=(http.urlquote(job_execution.cluster_id),))
            else:
                # No link should be generated for a deleted Cluster.
                return None

        def get_data(self, job_execution):
            return job_execution.cluster_name or _("Not available")

    class DurationColumn(tables.Column):

        def get_data(self, job_execution):
            status = job_execution.info["status"]

            end_time = None
            if status in [k for k, v in JobsTable.STATUS_CHOICES]:
                end_time = job_execution.updated_at

            return helpers.Helpers(None).get_duration(job_execution.created_at,
                                                      end_time)

    STATUS_CHOICES = (
        ("DONEWITHERROR", False),
        ("FAILED", False),
        ("KILLED", False),
        ("SUCCEEDED", True),
    )
    STATUS_DISPLAY_CHOICES = (
        ("DONEWITHERROR", pgettext_lazy("Current status of a Job",
                                        u"Done with Error")),
        ("FAILED", pgettext_lazy("Current status of a Job",
                                 u"Failed")),
        ("KILLED", pgettext_lazy("Current status of a Job",
                                 u"Killed")),
        ("SUCCEEDED", pgettext_lazy("Current status of a Job",
                                    u"Succeeded")),
    )

    name = tables.Column("id",
                         verbose_name=_("ID"),
                         display_choices=(("id", "ID"),
                                          ("name", pgettext_lazy("Name")),),
                         link=("horizon:project:data_processing."
                               "jobs:details"))
    job_name = JobNameColumn(
        "job_name",
        verbose_name=_("Job Template"),
        link=JobNameColumn.link)

    cluster_name = ClusterNameColumn(
        "cluster_name",
        verbose_name=_("Cluster"),
        link=ClusterNameColumn.link)

    status = StatusColumn("info",
                          status=True,
                          status_choices=STATUS_CHOICES,
                          display_choices=STATUS_DISPLAY_CHOICES,
                          verbose_name=_("Status"))

    duration = DurationColumn("duration",
                              verbose_name=_("Duration"))

    def get_object_display(self, datum):
        return datum.id

    class Meta(object):
        name = "jobs"
        row_class = UpdateRow
        status_columns = ["status"]
        verbose_name = _("Jobs")
        table_actions = [JobExecutionGuide,
                         DeleteJob,
                         JobsFilterAction]

        table_actions_menu = [MakePublic, MakePrivate,
                              MakeProtected, MakeUnProtected]
        row_actions = [DeleteJob,
                       ReLaunchJobExistingCluster,
                       ReLaunchJobNewCluster,
                       MakePublic, MakePrivate,
                       MakeProtected, MakeUnProtected]



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy
from saharaclient.api import base as api_base

from horizon import tables
from horizon.tabs import base as tabs_base

from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing \
    import tables as sahara_table
from sahara_dashboard.content.data_processing.utils \
    import acl as acl_utils


class CreateJobBinary(tables.LinkAction):
    name = "create"
    verbose_name = _("Create Job Binary")
    url = "horizon:project:data_processing.jobs:create-job-binary"
    classes = ("ajax-modal",)
    icon = "plus"


class DeleteJobBinary(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Job Binary",
            u"Delete Job Binaries",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Job Binary",
            u"Deleted Job Binaries",
            count
        )

    def delete(self, request, obj_id):
        jb = saharaclient.job_binary_get(request, obj_id)

        url_parts = jb.url.split("://")
        jb_type = url_parts[0]
        jb_internal_id = url_parts[len(url_parts) - 1]

        if jb_type == "internal-db":
            try:
                saharaclient.job_binary_internal_delete(request,
                                                        jb_internal_id)
            except api_base.APIException:
                # nothing to do for job-binary-internal if
                # it does not exist.
                pass

        saharaclient.job_binary_delete(request, obj_id)


class DownloadJobBinary(tables.LinkAction):
    name = "download_job_binary"
    verbose_name = _("Download Job Binary")
    url = "horizon:project:data_processing.jobs:download"
    classes = ("btn-edit",)


class EditJobBinary(tables.LinkAction):
    name = "edit_job_binary"
    verbose_name = _("Edit Job Binary")
    url = "horizon:project:data_processing.jobs:edit-job-binary"
    classes = ("btn-edit", "ajax-modal",)


class MakePublic(acl_utils.MakePublic):
    def change_rule_method(self, request, datum_id, **update_kwargs):
        saharaclient.job_binary_update(request, datum_id, update_kwargs)


class MakePrivate(acl_utils.MakePrivate):
    def change_rule_method(self, request, datum_id, **update_kwargs):
        saharaclient.job_binary_update(request, datum_id, update_kwargs)


class MakeProtected(acl_utils.MakeProtected):
    def change_rule_method(self, request, datum_id, **update_kwargs):
        saharaclient.job_binary_update(request, datum_id, update_kwargs)


class MakeUnProtected(acl_utils.MakeUnProtected):
    def change_rule_method(self, request, datum_id, **update_kwargs):
        saharaclient.job_binary_update(request, datum_id, update_kwargs)


class JobBinariesTable(sahara_table.SaharaPaginateTabbedTable):
    tab_name = 'job_tabs%sjob_binaries_tab' % tabs_base.SEPARATOR
    name = tables.Column(
        "name",
        verbose_name=_("Name"),
        link="horizon:project:data_processing.jobs:jb-details")
    type = tables.Column("url",
                         verbose_name=_("Url"))
    description = tables.Column("description",
                                verbose_name=_("Description"))

    class Meta(object):
        name = "job_binaries"
        verbose_name = _("Job Binaries")
        table_actions = (CreateJobBinary,
                         DeleteJobBinary,)
        table_actions_menu = (MakePublic,
                              MakePrivate,
                              MakeProtected,
                              MakeUnProtected)
        row_actions = (DeleteJobBinary, DownloadJobBinary, EditJobBinary,
                       MakePublic, MakePrivate, MakeProtected,
                       MakeUnProtected)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from horizon import tables

import sahara_dashboard.api.sahara as saharaclient


class SaharaPaginateTabbedTable(tables.DataTable):
    def get_pagination_string(self):
        return "tab=%s&limit=%s&marker=%s" % (
            self.tab_name,
            saharaclient.get_page_size(self.request),
            self.data.next
        )

    def get_prev_pagination_string(self):
        if self.data.prev is None:
            return "tab=%s&limit=%s" % (
                self.tab_name,
                saharaclient.get_page_size(self.request)
            )
        return "tab=%s&limit=%s&marker=%s" % (
            self.tab_name,
            saharaclient.get_page_size(self.request),
            self.data.prev
        )

    def has_more_data(self):
        return hasattr(self.data, 'next') and self.data.next is not None

    def has_prev_data(self):
        if hasattr(self.data, 'prev'):
            if 'tab' in self.request.GET:
                if self.tab_name == self.request.GET['tab']:
                    if 'marker' not in self.request.GET:
                        return False
                    else:
                        return True
        return False



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from openstack_dashboard.api import neutron
from openstack_dashboard.dashboards.project.instances \
    import utils as nova_utils
from oslo_serialization import jsonutils as json
from saharaclient.api import base as api_base

from sahara_dashboard.api import sahara as saharaclient

BASE_IMAGE_URL = "horizon:project:data_processing.clusters:register"


class ImportNodegroupTemplateFileForm(forms.SelfHandlingForm):

    class Meta(object):
        name = _("Import Node Group Template")

    def __init__(self, *args, **kwargs):
        self.next_view = kwargs.pop('next_view')
        super(ImportNodegroupTemplateFileForm, self).__init__(
            *args, **kwargs)

    template_upload = forms.FileField(
        label=_('Template File'),
        required=True)

    def handle(self, request, data):
        kwargs = {'template_upload': data['template_upload']}
        request.method = 'GET'
        return self.next_view.as_view()(request, **kwargs)


class ImportNodegroupTemplateDetailsForm(forms.SelfHandlingForm):

    class Meta(object):
        name = _("Import Node Group Template")

    template = forms.CharField(
        widget=forms.widgets.HiddenInput)

    name = forms.CharField(label=_("Name"),
                           required=False,
                           help_text=_("Name must be provided "
                                       "either here or in the template. If "
                                       "provided in both places, this one "
                                       "will be used."))

    security_groups = forms.MultipleChoiceField(
        label=_("Security Groups"),
        widget=forms.CheckboxSelectMultiple(),
        help_text=_("Launch instances in these security groups. "
                    "Auto security group will be determined by the "
                    "value present in the imported template."),
        required=False)

    floating_ip_pool = forms.ChoiceField(
        label=_("Floating IP Pool"),
        required=False)

    flavor = forms.ChoiceField(label=_("OpenStack Flavor"))

    image_id = forms.DynamicChoiceField(label=_("Base Image"),
                                        add_item_link=BASE_IMAGE_URL)

    def _populate_image_choices(self, request, plugin, hadoop_version):
        all_images = saharaclient.image_list(request)
        details = saharaclient.plugin_get_version_details(request,
                                                          plugin,
                                                          hadoop_version)
        return [(image.id, image.name) for image in all_images
                if (set(details.required_image_tags).
                issubset(set(image.tags)))]

    def __init__(self, *args, **kwargs):
        try:
            request = args[0]
            template_string = ""

            if "template_upload" in kwargs:
                template_upload = kwargs.pop('template_upload')
                super(ImportNodegroupTemplateDetailsForm, self).__init__(
                    *args, **kwargs)

                template_string = template_upload.read()
                self.fields["template"].initial = template_string

            else:
                super(ImportNodegroupTemplateDetailsForm, self).__init__(
                    *args, **kwargs)
                template_string = self.data["template"]

            template_json = json.loads(template_string)
            template_json = template_json["node_group_template"]

            security_group_list = neutron.security_group_list(request)
            security_group_choices = \
                [(sg.id, sg.name) for sg in security_group_list]
            self.fields["security_groups"].choices = security_group_choices

            pools = neutron.floating_ip_pools_list(request)
            pool_choices = [(pool.id, pool.name) for pool in pools]
            pool_choices.insert(0, (None, "Do not assign floating IPs"))
            self.fields["floating_ip_pool"].choices = pool_choices

            flavors = nova_utils.flavor_list(request)
            if flavors:
                self.fields["flavor"].choices = nova_utils.sort_flavor_list(
                    request, flavors)
            else:
                self.fields["flavor"].choices = []

            self.fields["image_id"].choices = \
                self._populate_image_choices(request,
                                             template_json["plugin_name"],
                                             template_json["hadoop_version"])
        except (ValueError, KeyError):
            raise exceptions.BadRequest(_("Could not parse template"))
        except Exception:
            exceptions.handle(request)

    def handle(self, request, data):
        try:
            template = data["template"]
            template = json.loads(template)
            template = template["node_group_template"]

            if not data["name"] and "name" not in template.keys():
                return False
            if data["name"]:
                template["name"] = data["name"]

            template["security_groups"] = data["security_groups"]
            template["floating_ip_pool"] = data["floating_ip_pool"]
            template["flavor_id"] = data["flavor"]
            template["image_id"] = data["image_id"]

            saharaclient.nodegroup_template_create(request, **template)
            return True
        except api_base.APIException as e:
            self.error_description = str(e)
            return False
        except Exception as e:
            if isinstance(e, TypeError):
                raise exceptions.BadRequest(
                    _("Template JSON contained invalid key"))
            else:
                raise exceptions.BadRequest(_("Could not parse template"))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django import http
from django.urls import reverse
from mox3.mox import IsA  # noqa
import six

from sahara_dashboard import api
from sahara_dashboard.test import helpers as test


INDEX_URL = reverse('horizon:project:data_processing.jobs:index')
DETAILS_URL = reverse(
    'horizon:project:data_processing.jobs:details', args=['id'])


class DataProcessingJobTests(test.TestCase):
    @test.create_stubs({api.sahara: ('job_execution_list',
                                     'plugin_list', 'job_binary_list',
                                     'data_source_list',
                                     'job_list')})
    def test_index(self):
        api.sahara.job_execution_list(IsA(http.HttpRequest), {}) \
            .AndReturn(self.job_executions.list())
        self.mox.ReplayAll()
        res = self.client.get(INDEX_URL)
        self.assertEqual(
            "cluster-1",
            (res.context_data["tab_group"]._tabs["jobs_tab"].
             _tables["jobs"].data[0].cluster_name))
        self.assertEqual(
            "job-1",
            (res.context_data["tab_group"]._tabs["jobs_tab"].
             _tables["jobs"].data[0].job_name))
        self.assertTemplateUsed(res, 'jobs/index.html')
        self.assertContains(res, 'Jobs')

    @test.create_stubs({api.sahara: ('job_execution_get',
                                     'cluster_get', 'job_get',
                                     'data_source_get')})
    def test_details(self):
        api.sahara.job_execution_get(IsA(http.HttpRequest), IsA(six.text_type)) \
            .MultipleTimes().AndReturn(self.job_executions.first())
        self.mox.ReplayAll()
        res = self.client.get(DETAILS_URL)
        self.assertTemplateUsed(res, 'horizon/common/_detail.html')
        self.assertContains(res, 'RUNNING')

    @test.create_stubs({api.sahara: ('job_execution_list',
                                     'job_execution_delete')})
    def test_delete(self):
        job_exec = self.job_executions.first()
        api.sahara.job_execution_list(IsA(http.HttpRequest), {}) \
            .AndReturn(self.job_executions.list())
        api.sahara.job_execution_delete(IsA(http.HttpRequest), job_exec.id)
        self.mox.ReplayAll()

        form_data = {'action': 'jobs__delete__%s' % job_exec.id}
        res = self.client.post(INDEX_URL, form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)
        self.assertMessageCount(success=1)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import uuid

from django.forms import widgets
from django import template
from django.template import defaultfilters
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
import six

from horizon import exceptions
from horizon import forms
from horizon import messages

from sahara_dashboard.api import manila as manilaclient
from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing.utils \
    import acl as acl_utils
from saharaclient.api import base


class LabeledInput(widgets.TextInput):

    def __init__(self, jb_type, *args, **kwargs):
        self.jb_type = jb_type
        super(LabeledInput, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        input = super(LabeledInput, self).render(name, value, attrs)
        label = "<span id='%s'>%s</span>" %\
            ("id_%s_label" % name,
             "%s://" % self.jb_type)
        result = "%s%s" % (label, input)
        return mark_safe(result)


class JobBinaryCreateForm(forms.SelfHandlingForm):
    NEW_SCRIPT = "newscript"
    UPLOAD_BIN = "uploadfile"
    action_url = ('horizon:project:data_processing.'
                  'jobs:create-job-binary')

    def __init__(self, request, *args, **kwargs):
        super(JobBinaryCreateForm, self).__init__(request, *args, **kwargs)

        self.help_text_template = "job_binaries/_create_job_binary_help.html"

        self.fields["job_binary_name"] = forms.CharField(label=_("Name"))

        self.fields["job_binary_type"] = forms.ChoiceField(
            label=_("Storage type"),
            widget=forms.Select(
                attrs={
                    'class': 'switchable',
                    'data-slug': 'jb_type'
                }))

        self.fields["job_binary_url"] = forms.CharField(
            label=_("URL"),
            required=False,
            widget=LabeledInput(
                "swift",
                attrs={
                    'class': 'switched',
                    'data-switch-on': 'jb_type',
                    'data-jb_type-swift': _('URL')
                }))

        self.fields["job_binary_manila_share"] = forms.ChoiceField(
            label=_("Share"),
            required=False,
            widget=forms.Select(
                attrs={
                    'class': 'switched',
                    'data-switch-on': 'jb_type',
                    'data-jb_type-manila': _('Share')
                }))

        self.fields["job_binary_manila_path"] = forms.CharField(
            label=_("Path"),
            required=False,
            widget=forms.TextInput(
                attrs={
                    'class': 'switched',
                    'data-switch-on': 'jb_type',
                    'data-jb_type-manila': _('Path')
                }))

        self.fields["job_binary_internal"] = forms.ChoiceField(
            label=_("Internal binary"),
            required=False,
            widget=forms.Select(
                attrs={
                    'class': 'switched switchable',
                    'data-slug': 'jb_internal',
                    'data-switch-on': 'jb_type',
                    'data-jb_type-internal-db': _('Internal Binary')
                }))

        self.fields["job_binary_file"] = forms.FileField(
            label=_("Upload File"),
            required=False,
            widget=forms.ClearableFileInput(
                attrs={
                    'class': 'switched',
                    'data-switch-on': 'jb_internal',
                    'data-jb_internal-uploadfile': _("Upload File")
                }))

        self.fields["job_binary_script_name"] = forms.CharField(
            label=_("Script name"),
            required=False,
            widget=forms.TextInput(
                attrs={
                    'class': 'switched',
                    'data-switch-on': 'jb_internal',
                    'data-jb_internal-newscript': _("Script name")
                }))

        self.fields["job_binary_script"] = forms.CharField(
            label=_("Script text"),
            required=False,
            widget=forms.Textarea(
                attrs={
                    'rows': 4,
                    'class': 'switched',
                    'data-switch-on': 'jb_internal',
                    'data-jb_internal-newscript': _("Script text")
                }))

        self.fields["job_binary_username"] = forms.CharField(
            label=_("Username"),
            required=False,
            widget=forms.TextInput(
                attrs={
                    'class': 'switched',
                    'data-switch-on': 'jb_type',
                    'data-jb_type-swift': _('Username')
                }))

        self.fields["job_binary_password"] = forms.CharField(
            label=_("Password"),
            required=False,
            widget=forms.PasswordInput(
                attrs={
                    'autocomplete': 'off',
                    'class': 'switched',
                    'data-switch-on': 'jb_type',
                    'data-jb_type-swift': _('Password')
                }))

        self.fields["job_binary_s3_url"] = forms.CharField(
            label=_("URL"),
            required=False,
            widget=LabeledInput(
                "s3",
                attrs={
                    'class': 'switched',
                    'data-switch-on': 'jb_type',
                    'data-jb_type-s3': _('URL')
                }))

        self.fields["job_binary_s3_endpoint"] = forms.CharField(
            label=_("S3 Endpoint"),
            required=False,
            widget=forms.TextInput(
                attrs={
                    'class': 'switched',
                    'data-switch-on': 'jb_type',
                    'data-jb_type-s3': _('S3 Endpoint')
                }))

        self.fields["job_binary_access_key"] = forms.CharField(
            label=_("Access Key"),
            required=False,
            widget=forms.TextInput(
                attrs={
                    'class': 'switched',
                    'data-switch-on': 'jb_type',
                    'data-jb_type-s3': _('Access Key')
                }))

        self.fields["job_binary_secret_key"] = forms.CharField(
            label=_("Secret Key"),
            required=False,
            widget=forms.PasswordInput(
                attrs={
                    'autocomplete': 'off',
                    'class': 'switched',
                    'data-switch-on': 'jb_type',
                    'data-jb_type-s3': _('Secret Key')
                }))

        self.fields["job_binary_description"] = (
            forms.CharField(label=_("Description"),
                            required=False,
                            widget=forms.Textarea()))

        self.fields["is_public"] = acl_utils.get_is_public_form(
            _("job binary"))
        self.fields["is_protected"] = acl_utils.get_is_protected_form(
            _("job binary"))

        self.fields["job_binary_type"].choices =\
            [("internal-db", "Internal database"),
             ("swift", "Swift"),
             ("s3", "S3")]

        self.fields["job_binary_internal"].choices =\
            self.populate_job_binary_internal_choices(request)

        if saharaclient.base.is_service_enabled(request, 'share'):
            self.fields["job_binary_type"].choices.append(("manila", "Manila"))
            self.fields["job_binary_manila_share"].choices = (
                self.populate_job_binary_manila_share_choices(request))

        self.load_form_values()

    def load_form_values(self):
        if "job_binary" in self.initial:
            jb = self.initial["job_binary"]
            for field in self.fields:
                if self.FIELD_MAP[field]:
                    if field in ["job_binary_url", "job_binary_s3_url"]:
                        url = getattr(jb, self.FIELD_MAP[field], None)
                        self.set_initial_values_by_type(url)
                    else:
                        self.fields[field].initial = (
                            getattr(jb, self.FIELD_MAP[field], None))

    def set_initial_values_by_type(self, url):
        parsed = six.moves.urllib.parse.urlparse(url)
        self.fields["job_binary_type"].initial = parsed.scheme
        if parsed.scheme == "manila":
            self.fields["job_binary_manila_share"].initial = parsed.netloc
            self.fields["job_binary_manila_path"].initial = parsed.path
        elif parsed.scheme == "swift":
            self.fields["job_binary_url"].initial = (
                "{0}{1}".format(parsed.netloc, parsed.path))
        elif parsed.scheme == "s3":
            self.fields["job_binary_s3_url"].initial = (
                "{0}{1}".format(parsed.netloc, parsed.path))
        else:
            self.fields["job_binary_url"].initial = "{0}".format(parsed.netloc)

    def populate_job_binary_manila_share_choices(self, request):
        try:
            shares = manilaclient.share_list(request)
            choices = [(s.id, s.name) for s in shares]
        except Exception:
            exceptions.handle(request, _("Failed to get list of shares"))
            choices = []
        return choices

    def populate_job_binary_internal_choices(self, request):
        try:
            job_binaries = saharaclient.job_binary_internal_list(request)
        except Exception:
            exceptions.handle(request,
                              _("Failed to get list of internal binaries."))
            job_binaries = []

        choices = [(job_binary.id, job_binary.name)
                   for job_binary in job_binaries]
        choices.insert(0, (self.NEW_SCRIPT, '*Create a script'))
        choices.insert(0, (self.UPLOAD_BIN, '*Upload a new file'))

        return choices

    def handle(self, request, context):
        try:
            extra = {}
            jb_type = context.get("job_binary_type")

            if jb_type != "s3":
                bin_url = "%s://%s" % (context["job_binary_type"],
                                       context["job_binary_url"])
            else:
                bin_url = "%s://%s" % (context["job_binary_type"],
                                       context["job_binary_s3_url"])
            if(jb_type == "internal-db"):
                bin_url = self.handle_internal(request, context)
            elif(jb_type == "swift"):
                extra = self.handle_swift(request, context)
            elif(jb_type == "s3"):
                extra = self.handle_s3(request, context)
            elif(jb_type == "manila"):
                bin_url = "%s://%s%s" % (context["job_binary_type"],
                                         context["job_binary_manila_share"],
                                         context["job_binary_manila_path"])

            bin_object = saharaclient.job_binary_create(
                request,
                context["job_binary_name"],
                bin_url,
                context["job_binary_description"],
                extra,
                is_public=context['is_public'],
                is_protected=context['is_protected']
            )
            messages.success(request, "Successfully created job binary")
            return bin_object
        except base.APIException as ex:

            exceptions.handle(request, _("Unable to create job binary: %s") %
                              ex.error_message)
            return False

        except Exception:
            exceptions.handle(request, _("Unable to create job binary"))
            return False

    def get_help_text(self, extra_context=None):
        text = ""
        extra_context = extra_context or {}
        if self.help_text_template:
            tmpl = template.loader.get_template(self.help_text_template)
            text += tmpl.render(extra_context, self.request)
        else:
            text += defaultfilters.linebreaks(force_text(self.help_text))
        return defaultfilters.safe(text)

    class Meta(object):
        name = _("Create Job Binary")
        help_text_template = "job_binaries/_create_job_binary_help.html"

    def handle_internal(self, request, context):
        result = ""

        bin_id = context["job_binary_internal"]
        if(bin_id == self.UPLOAD_BIN):
            try:
                result = saharaclient.job_binary_internal_create(
                    request,
                    self.get_unique_binary_name(
                        request, request.FILES["job_binary_file"].name),
                    request.FILES["job_binary_file"].read())
                bin_id = result.id
            except Exception:
                exceptions.handle(request,
                                  _("Unable to upload job binary"))
                return None
        elif(bin_id == self.NEW_SCRIPT):
            try:
                result = saharaclient.job_binary_internal_create(
                    request,
                    self.get_unique_binary_name(
                        request, context["job_binary_script_name"]),
                    context["job_binary_script"])
                bin_id = result.id
            except base.APIException as ex:

                exceptions.handle(
                    request,
                    _("Unable to create job binary: %s") % ex.error_message)
                return None

            except Exception:
                exceptions.handle(request, _("Unable to create job binary"))
                return None

        return "internal-db://%s" % bin_id

    def handle_swift(self, request, context):
        username = context["job_binary_username"]
        password = context["job_binary_password"]

        extra = {
            "user": username,
            "password": password
        }
        return extra

    def handle_s3(self, request, context):
        accesskey = context['job_binary_access_key']
        secretkey = context['job_binary_secret_key']
        endpoint = context['job_binary_s3_endpoint']

        extra = {}
        if accesskey != "":
            extra["accesskey"] = accesskey
        if secretkey != "":
            extra["secretkey"] = secretkey
        if endpoint != "":
            extra["endpoint"] = endpoint

        return extra

    def get_unique_binary_name(self, request, base_name):
        try:
            internals = saharaclient.job_binary_internal_list(request)
        except Exception:
            internals = []
            exceptions.handle(request,
                              _("Failed to fetch internal binary list"))
        names = [internal.name for internal in internals]
        if base_name in names:
            return "%s_%s" % (base_name, uuid.uuid1())
        return base_name


class JobBinaryEditForm(JobBinaryCreateForm):
    FIELD_MAP = {
        'job_binary_description': 'description',
        'job_binary_file': None,
        'job_binary_internal': None,
        'job_binary_name': 'name',
        'job_binary_password': None,
        'job_binary_script': None,
        'job_binary_script_name': None,
        'job_binary_type': None,
        'job_binary_url': 'url',
        'job_binary_s3_url': 'url',
        'job_binary_username': None,
        'job_binary_manila_share': None,
        'job_binary_manila_path': None,
        'job_binary_access_key': None,
        'job_binary_secret_key': None,
        'job_binary_s3_endpoint': None,
        'is_public': 'is_public',
        'is_protected': 'is_protected',
    }

    def handle(self, request, context):
        try:
            extra = {}
            if context["job_binary_type"] != "s3":
                bin_url = "%s://%s" % (context["job_binary_type"],
                                       context["job_binary_url"])
            else:
                bin_url = "%s://%s" % (context["job_binary_type"],
                                       context["job_binary_s3_url"])

            if (context["job_binary_type"] == "swift"):
                extra = self.handle_swift(request, context)

            if (context["job_binary_type"] == "s3"):
                extra = self.handle_s3(request, context)

            update_data = {
                "name": context["job_binary_name"],
                "description": context["job_binary_description"],
                "extra": extra,
                "url": bin_url,
                'is_public': context['is_public'],
                'is_protected': context['is_protected']
            }

            bin_object = saharaclient.job_binary_update(
                request, self.initial["job_binary"].id, update_data)

            messages.success(request, "Successfully updated job binary")
            return bin_object
        except base.APIException as ex:

            exceptions.handle(request, _("Unable to update job binary: %s") %
                              ex.error_message)
            return False

        except Exception:
            exceptions.handle(request, _("Unable to update job binary"))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django import http
from django.urls import reverse
import mock
from mox3.mox import IgnoreArg  # noqa
from mox3.mox import IsA  # noqa
import six

from openstack_dashboard import api as dash_api

from sahara_dashboard import api
from sahara_dashboard.content.data_processing.utils \
    import workflow_helpers
from sahara_dashboard.content.data_processing.clusters. \
    nodegroup_templates.workflows import create as create_workflow
from sahara_dashboard.test import helpers as test


INDEX_URL = reverse(
    'horizon:project:data_processing.clusters:nodegroup-templates-tab')
DETAILS_URL = reverse(
    'horizon:project:data_processing.clusters:details',
    args=['id'])
CREATE_URL = reverse(
    'horizon:project:data_processing.clusters:' +
    'configure-nodegroup-template')


class DataProcessingNodeGroupTests(test.TestCase):
    @mock.patch('openstack_dashboard.api.base.is_service_enabled')
    def _setup_copy_test(self, service_checker):
        service_checker.return_value = True
        ngt = self.nodegroup_templates.first()
        configs = self.plugins_configs.first()
        dash_api.cinder.extension_supported(IsA(http.HttpRequest),
                                            'AvailabilityZones') \
            .AndReturn(True)
        dash_api.cinder.availability_zone_list(IsA(http.HttpRequest))\
            .AndReturn(self.availability_zones.list())
        dash_api.cinder.volume_type_list(IsA(http.HttpRequest))\
            .AndReturn([])
        api.sahara.nodegroup_template_get(IsA(http.HttpRequest),
                                          ngt.id) \
            .AndReturn(ngt)
        api.sahara.plugin_get_version_details(IsA(http.HttpRequest),
                                              ngt.plugin_name,
                                              ngt.hadoop_version) \
            .MultipleTimes().AndReturn(configs)
        dash_api.neutron.floating_ip_pools_list(IsA(http.HttpRequest)) \
            .AndReturn([])
        dash_api.neutron.security_group_list(IsA(http.HttpRequest)) \
            .AndReturn([])

        self.mox.ReplayAll()

        url = reverse(
            'horizon:project:data_processing.clusters:copy',
            args=[ngt.id])
        res = self.client.get(url)

        return ngt, configs, res

    @test.create_stubs({api.sahara: ('cluster_template_list',
                                     'image_list',
                                     'cluster_list',
                                     'nodegroup_template_list')})
    def test_index(self):
        api.sahara.nodegroup_template_list(IsA(http.HttpRequest), {}) \
            .AndReturn(self.nodegroup_templates.list())
        self.mox.ReplayAll()
        res = self.client.get(INDEX_URL +
                              "?tab=cluster_tabs__node_group_templates_tab")
        self.assertTemplateUsed(res, 'clusters/index.html')
        self.assertContains(res, 'Node Group Templates')
        self.assertContains(res, 'Name')
        self.assertContains(res, 'Plugin')

    @test.create_stubs({api.sahara: ('nodegroup_template_get',),
                        dash_api.nova: ('flavor_get',)})
    def test_details(self):
        flavor = self.flavors.first()
        ngt = self.nodegroup_templates.first()
        dash_api.nova.flavor_get(IsA(http.HttpRequest), flavor.id) \
            .AndReturn(flavor)
        api.sahara.nodegroup_template_get(IsA(http.HttpRequest),
                                          IsA(six.text_type)) \
            .MultipleTimes().AndReturn(ngt)
        self.mox.ReplayAll()
        res = self.client.get(DETAILS_URL)
        self.assertTemplateUsed(res, 'horizon/common/_detail.html')
        self.assertContains(res, 'sample-template')

    @test.create_stubs({api.sahara: ('nodegroup_template_list',
                                     'nodegroup_template_delete')})
    def test_delete(self):
        ngt = self.nodegroup_templates.first()
        api.sahara.nodegroup_template_list(IsA(http.HttpRequest), {}) \
            .AndReturn(self.nodegroup_templates.list())
        api.sahara.nodegroup_template_delete(IsA(http.HttpRequest), ngt.id)
        self.mox.ReplayAll()

        form_data = {'action': 'nodegroup_templates__delete__%s' % ngt.id}
        res = self.client.post(INDEX_URL, form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)
        self.assertMessageCount(success=1)

    @test.create_stubs({api.sahara: ('nodegroup_template_get',
                                     'plugin_get_version_details',
                                     'image_list'),
                        dash_api.nova: ('availability_zone_list',
                                        'flavor_list'),
                        dash_api.neutron: ('floating_ip_pools_list',
                                           'security_group_list'),
                        dash_api.cinder: ('extension_supported',
                                          'availability_zone_list',
                                          'volume_type_list')})
    def test_copy(self):
        ngt, configs, res = self._setup_copy_test()
        workflow = res.context['workflow']
        step = workflow.get_step("generalconfigaction")
        self.assertEqual(step.action['nodegroup_name'].field.initial,
                         ngt.name + "-copy")

    @test.create_stubs({api.sahara: ('client',
                                     'nodegroup_template_create',
                                     'plugin_get_version_details'),
                        dash_api.neutron: ('floating_ip_pools_list',
                                           'security_group_list'),
                        dash_api.nova: ('flavor_list',
                                        'availability_zone_list'),
                        dash_api.cinder: ('extension_supported',
                                          'availability_zone_list',
                                          'volume_type_list')})
    @mock.patch('openstack_dashboard.api.base.is_service_enabled')
    def test_create(self, service_checker):
        service_checker.return_value = True
        flavor = self.flavors.first()
        ngt = self.nodegroup_templates.first()
        configs = self.plugins_configs.first()
        new_name = ngt.name + '-new'
        self.mox.StubOutWithMock(
            workflow_helpers, 'parse_configs_from_context')

        dash_api.cinder.extension_supported(IsA(http.HttpRequest),
                                            'AvailabilityZones') \
            .AndReturn(True)
        dash_api.cinder.availability_zone_list(IsA(http.HttpRequest))\
            .AndReturn(self.availability_zones.list())
        dash_api.cinder.volume_type_list(IsA(http.HttpRequest))\
            .AndReturn([])
        dash_api.nova.flavor_list(IsA(http.HttpRequest)).AndReturn([flavor])
        api.sahara.plugin_get_version_details(IsA(http.HttpRequest),
                                              ngt.plugin_name,
                                              ngt.hadoop_version) \
            .MultipleTimes().AndReturn(configs)
        dash_api.neutron.floating_ip_pools_list(IsA(http.HttpRequest)) \
            .AndReturn([])
        dash_api.neutron.security_group_list(IsA(http.HttpRequest)) \
            .AndReturn([])
        workflow_helpers.parse_configs_from_context(
            IgnoreArg(), IgnoreArg()).AndReturn({})
        api.sahara.nodegroup_template_create(
            IsA(http.HttpRequest),
            **{'name': new_name,
               'plugin_name': ngt.plugin_name,
               'hadoop_version': ngt.hadoop_version,
               'description': ngt.description,
               'flavor_id': flavor.id,
               'volumes_per_node': None,
               'volumes_size': None,
               'volume_type': None,
               'volume_local_to_instance': False,
               'volumes_availability_zone': None,
               'node_processes': ['namenode'],
               'node_configs': {},
               'floating_ip_pool': None,
               'security_groups': [],
               'image_id': None,
               'auto_security_group': True,
               'availability_zone': None,
               'is_proxy_gateway': False,
               'use_autoconfig': True,
               'shares': [],
               'is_public': False,
               'is_protected': False})\
            .AndReturn(True)

        self.mox.ReplayAll()

        res = self.client.post(
            CREATE_URL,
            {'nodegroup_name': new_name,
             'plugin_name': ngt.plugin_name,
             ngt.plugin_name + '_version': '1.2.1',
             'hadoop_version': ngt.hadoop_version,
             'description': ngt.description,
             'flavor': flavor.id,
             'availability_zone': None,
             'storage': 'ephemeral_drive',
             'volumes_per_node': 0,
             'volumes_size': 0,
             'volume_type': None,
             'volume_local_to_instance': False,
             'volumes_availability_zone': None,
             'floating_ip_pool': None,
             'security_autogroup': True,
             'processes': 'HDFS:namenode',
             'use_autoconfig': True,
             'shares': [],
             'is_public': False,
             'is_protected': False})

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)
        self.assertMessageCount(success=1)

    @test.create_stubs({api.sahara: ('client',
                                     'nodegroup_template_create',
                                     'nodegroup_template_update',
                                     'nodegroup_template_get',
                                     'plugin_get_version_details'),
                        dash_api.neutron: ('floating_ip_pools_list',
                                           'security_group_list'),
                        dash_api.nova: ('flavor_list',
                                        'availability_zone_list'),
                        dash_api.cinder: ('extension_supported',
                                          'availability_zone_list',
                                          'volume_type_list')})
    @mock.patch('openstack_dashboard.api.base.is_service_enabled')
    def test_update(self, service_checker):
        service_checker.return_value = True
        flavor = self.flavors.first()
        ngt = self.nodegroup_templates.first()
        configs = self.plugins_configs.first()
        new_name = ngt.name + '-updated'
        UPDATE_URL = reverse(
            'horizon:project:data_processing.clusters:edit',
            kwargs={'template_id': ngt.id})
        self.mox.StubOutWithMock(
            workflow_helpers, 'parse_configs_from_context')

        dash_api.cinder.extension_supported(IsA(http.HttpRequest),
                                            'AvailabilityZones') \
            .AndReturn(True)
        dash_api.cinder.availability_zone_list(IsA(http.HttpRequest)) \
            .AndReturn(self.availability_zones.list())
        dash_api.cinder.volume_type_list(IsA(http.HttpRequest))\
            .AndReturn([])
        dash_api.nova.flavor_list(IsA(http.HttpRequest)).AndReturn([flavor])
        api.sahara.plugin_get_version_details(IsA(http.HttpRequest),
                                              ngt.plugin_name,
                                              ngt.hadoop_version) \
            .MultipleTimes().AndReturn(configs)
        dash_api.neutron.floating_ip_pools_list(IsA(http.HttpRequest)) \
            .AndReturn([])
        dash_api.neutron.security_group_list(IsA(http.HttpRequest)) \
            .AndReturn([])
        workflow_helpers.parse_configs_from_context(
            IgnoreArg(), IgnoreArg()).AndReturn({})
        api.sahara.nodegroup_template_get(IsA(http.HttpRequest),
                                          ngt.id) \
            .AndReturn(ngt)
        api.sahara.nodegroup_template_update(
            request=IsA(http.HttpRequest),
            ngt_id=ngt.id,
            name=new_name,
            plugin_name=ngt.plugin_name,
            hadoop_version=ngt.hadoop_version,
            flavor_id=flavor.id,
            description=ngt.description,
            volumes_per_node=0,
            volumes_size=None,
            volume_type=None,
            volume_local_to_instance=False,
            volumes_availability_zone=None,
            node_processes=['namenode'],
            node_configs={},
            floating_ip_pool=None,
            security_groups=[],
            auto_security_group=True,
            availability_zone=None,
            use_autoconfig=True,
            is_proxy_gateway=False,
            shares=[],
            is_protected=False,
            is_public=False,
            image_id=ngt.image_id).AndReturn(True)

        self.mox.ReplayAll()

        res = self.client.post(
            UPDATE_URL,
            {'ng_id': ngt.id,
             'nodegroup_name': new_name,
             'plugin_name': ngt.plugin_name,
             ngt.plugin_name + '_version': '1.2.1',
             'hadoop_version': ngt.hadoop_version,
             'description': ngt.description,
             'flavor': flavor.id,
             'availability_zone': None,
             'storage': 'ephemeral_drive',
             'volumes_per_node': 0,
             'volumes_size': 0,
             'volume_type': None,
             'volume_local_to_instance': False,
             'volumes_availability_zone': None,
             'floating_ip_pool': None,
             'is_proxy_gateway': False,
             'security_autogroup': True,
             'processes': 'HDFS:namenode',
             'use_autoconfig': True})

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)
        self.assertMessageCount(success=1)

    @test.create_stubs({api.sahara: ('nodegroup_template_get',
                                     'plugin_get_version_details',
                                     'image_list'),
                        dash_api.nova: ('availability_zone_list',
                                        'flavor_list'),
                        dash_api.neutron: ('floating_ip_pools_list',
                                           'security_group_list'),
                        dash_api.cinder: ('extension_supported',
                                          'availability_zone_list',
                                          'volume_type_list')})
    def test_workflow_steps(self):
        # since the copy workflow is the child of create workflow
        # it's better to test create workflow through copy workflow
        ngt, configs, res = self._setup_copy_test()
        workflow = res.context['workflow']
        expected_instances = [
            create_workflow.GeneralConfig,
            create_workflow.SelectNodeProcesses,
            create_workflow.SecurityConfig
        ]
        for expected, observed in zip(expected_instances, workflow.steps):
            self.assertIsInstance(observed, expected)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _
from oslo_serialization import jsonutils as json

from horizon import exceptions
from horizon import forms
from saharaclient.api import base as api_base

from sahara_dashboard.api import sahara as saharaclient
import sahara_dashboard.content.data_processing. \
    utils.workflow_helpers as whelpers
from sahara_dashboard import utils

BASE_IMAGE_URL = "horizon:project:data_processing.clusters:register"


class ImportClusterTemplateFileForm(forms.SelfHandlingForm):

    class Meta(object):
        name = _("Import Cluster Template")

    def __init__(self, *args, **kwargs):
        self.next_view = kwargs.pop("next_view")
        super(ImportClusterTemplateFileForm, self).__init__(
            *args, **kwargs)

    template_upload = forms.FileField(
        label=_("Template File"),
        required=True)

    def handle(self, request, data):
        kwargs = {"template_upload": data["template_upload"]}
        request.method = "GET"
        return self.next_view.as_view()(request, **kwargs)


class ImportClusterTemplateNameForm(forms.SelfHandlingForm):

    class Meta(object):
        name = _("Import Cluster Template")

    template_upload = forms.CharField(
        widget=forms.widgets.HiddenInput)

    name = forms.CharField(label=_("Name"),
                           required=False,
                           help_text=_("Name must be provided "
                                       "either here or in the template. If "
                                       "provided in both places, this one "
                                       "will be used."))

    def __init__(self, *args, **kwargs):
        try:
            request = args[0]
            template_string = ""
            self.next_view = kwargs.pop("next_view")

            if "template_upload" in kwargs:
                template_upload = kwargs.pop("template_upload")
                super(ImportClusterTemplateNameForm, self).__init__(
                    *args, **kwargs)

                template_string = template_upload.read()
                self.fields["template_upload"].initial = template_string

            else:
                super(ImportClusterTemplateNameForm, self).__init__(
                    *args, **kwargs)
        except (ValueError, KeyError):
            raise exceptions.BadRequest(_("Could not parse template"))
        except Exception:
            exceptions.handle(request)

    def handle(self, request, data):
        template = data["template_upload"]
        if data["name"]:
            template = json.loads(template)
            template["cluster_template"]["name"] = data["name"]
            template = json.dumps(template)
        kwargs = {"template_upload": template}
        request.method = "GET"
        return self.next_view.as_view()(request, **kwargs)


class ImportClusterTemplateNodegroupsForm(forms.SelfHandlingForm):

    class Meta(object):
        name = _("Import Cluster Template")

    template_upload = forms.CharField(
        widget=forms.widgets.HiddenInput)
    hidden_nodegroups_field = forms.CharField(
        required=False,
        widget=forms.HiddenInput(attrs={"class": "hidden_nodegroups_field"}))
    forms_ids = forms.CharField(
        required=False,
        widget=forms.HiddenInput())

    def __init__(self, *args, **kwargs):
        try:
            request = args[0]
            template_string = ""

            if "template_upload" in kwargs:
                template_string = kwargs.pop("template_upload")
                super(ImportClusterTemplateNodegroupsForm, self).__init__(
                    *args, **kwargs)

                self.fields["template_upload"].initial = template_string

            else:
                super(ImportClusterTemplateNodegroupsForm, self).__init__(
                    *args, **kwargs)
                template_string = self.data["template_upload"]

            template_json = json.loads(template_string)
            template_json = template_json["cluster_template"]

            req = request.GET.copy()
            req.update(request.POST)

            plugin = template_json["plugin_name"]
            version = template_json["hadoop_version"]

            if not plugin or not version:
                self.templates = saharaclient.nodegroup_template_find(request)
            else:
                self.templates = saharaclient.nodegroup_template_find(
                    request, plugin_name=plugin, hadoop_version=version)

            deletable = req.get("deletable", dict())

            if "forms_ids" in req:
                self.groups = []
                for id in json.loads(req["forms_ids"]):
                    group_name = "group_name_" + str(id)
                    template_id = "template_id_" + str(id)
                    count = "count_" + str(id)
                    serialized = "serialized_" + str(id)
                    self.groups.append({"name": req[group_name],
                                        "template_id": req[template_id],
                                        "count": req[count],
                                        "id": id,
                                        "deletable": deletable.get(
                                            req[group_name], "true"),
                                        "serialized": req[serialized]})

                    whelpers.build_node_group_fields(self,
                                                     group_name,
                                                     template_id,
                                                     count,
                                                     serialized)

        except (ValueError, KeyError):
            raise exceptions.BadRequest(_("Could not parse template"))
        except Exception:
            exceptions.handle(request)

    def handle(self, request, data):
        try:
            template = data["template_upload"]
            template = json.loads(template)
            template = template["cluster_template"]

            if "name" not in template.keys():
                return False

            if "neutron_management_network" in template:
                template["net_id"] = (
                    template.pop("neutron_management_network"))

            # default_image_id is not supported by the client now
            if "default_image_id" in template:
                template.pop("default_image_id")

            node_groups = []
            ids = json.loads(data['forms_ids'])
            for id in ids:
                name = data['group_name_' + str(id)]
                template_id = data['template_id_' + str(id)]
                count = data['count_' + str(id)]

                raw_ng = data.get("serialized_" + str(id))

                if raw_ng and raw_ng != 'null':
                    ng = json.loads(utils.deserialize(str(raw_ng)))
                else:
                    ng = dict()
                ng["name"] = name
                ng["count"] = count
                if template_id and template_id != u'None':
                    ng["node_group_template_id"] = template_id
                node_groups.append(ng)

            template["node_groups"] = node_groups

            saharaclient.cluster_template_create(request, **template)
            return True
        except api_base.APIException as e:
            self.error_description = str(e)
            return False
        except Exception as e:
            if isinstance(e, TypeError):
                raise exceptions.BadRequest(
                    _("Template JSON contained invalid key"))
            else:
                raise exceptions.BadRequest(_("Could not parse template"))
